@@ -20,7 +20,6 @@
 #include "nanonext.h"
 
 SEXP nano_AioSymbol;
-SEXP nano_AioHttpSymbol;
 SEXP nano_ContextSymbol;
 SEXP nano_CvSymbol;
 SEXP nano_DataSymbol;
@@ -37,7 +36,6 @@ SEXP nano_SocketSymbol;
 SEXP nano_StateSymbol;
 SEXP nano_StatusSymbol;
 SEXP nano_StreamSymbol;
-SEXP nano_TextframesSymbol;
 SEXP nano_TlsSymbol;
 SEXP nano_UrlSymbol;
 SEXP nano_ValueSymbol;
@@ -50,9 +48,7 @@ SEXP nano_error;
 SEXP nano_ncurlAio;
 SEXP nano_ncurlSession;
 SEXP nano_recvAio;
-SEXP nano_refHookIn;
-SEXP nano_refHookOut;
-SEXP nano_refList;
+SEXP nano_refHook;
 SEXP nano_sendAio;
 SEXP nano_success;
 SEXP nano_unresolved;
@@ -79,7 +75,6 @@ static void RegisterSymbols(void) {
   nano_StateSymbol = Rf_install("state");
   nano_StatusSymbol = Rf_install("status");
   nano_StreamSymbol = Rf_install("stream");
-  nano_TextframesSymbol = Rf_install("textframes");
   nano_TlsSymbol = Rf_install("tls");
   nano_UrlSymbol = Rf_install("url");
   nano_ValueSymbol = Rf_install("value");
@@ -112,18 +107,11 @@ static void PreserveObjects(void) {
   R_PreserveObject(nano_success = Rf_ScalarInteger(0));
   R_PreserveObject(nano_unresolved = Rf_shallow_duplicate(Rf_ScalarLogical(NA_LOGICAL)));
   Rf_classgets(nano_unresolved, Rf_mkString("unresolvedValue"));
-  nano_refHookIn = R_NilValue;
-  nano_refHookOut = R_NilValue;
-  nano_refList = R_NilValue;
+  R_PreserveObject(nano_refHook = Rf_list2(R_NilValue, R_NilValue));
 }
 
 static void ReleaseObjects(void) {
-  if (nano_refList != R_NilValue)
-    R_ReleaseObject(nano_refList);
-  if (nano_refHookOut != R_NilValue)
-    R_ReleaseObject(nano_refHookOut);
-  if (nano_refHookIn != R_NilValue)
-    R_ReleaseObject(nano_refHookIn);
+  R_ReleaseObject(nano_refHook);
   R_ReleaseObject(nano_unresolved);
   R_ReleaseObject(nano_success);
   R_ReleaseObject(nano_sendAio);
