@@ -76,14 +76,14 @@ nanotestz(n1$send(c("test", "", "spec"), mode = "raw", block = 500))
 nanotest(identical(n$recv("character", block = 500), c("test", "", "spec")))
 nanotestz(n$send(c(1L, 0L, 0L), mode = "r"))
 nanotest(length(n1$recv("int", block = 500)) == 3L)
-nanotestaio(saio <- n1$send_aio(random(1e3L), mode = 3L, timeout = 900))
+nanotestaio(saio <- n1$send_aio(Reduce(strcat, replicate(5, random(1e3L))), mode = 3L, timeout = 900))
 nanotestp(saio)
 nanotestaio(call_aio(saio))
 nanotestz(saio$result)
 nanotesterr(n$send("wrong mode", mode = "none"), "mode")
 nanotestaio(raio <- n$recv_aio(timeout = 500))
 nanotestp(raio)
-nanotest(nchar(call_aio(raio)[["value"]]) == 2000L)
+nanotest(nchar(call_aio(raio)[["value"]]) == 10000L)
 raio$newfield <- "doesnotwork"
 raio[["newfield"]] <- "doesnotwork"
 nanotestn(raio$newfield)
@@ -207,6 +207,7 @@ nanotestn(stop_aio(err))
 nanotesterr(err$result, "active")
 nanotestaio(err <- send_aio(ctx, "test"))
 nanotest(is_error_value(call_aio(err)$result))
+nanotest(is_error_value(call_aio_(err)$result))
 nanotest(is.integer(req$send(as.raw(1L), mode = 2L, block = 500)))
 nanotestaio(msg <- call_aio(recv_aio(ctx, mode = 8L, timeout = 500)))
 nanotestaio(saio <- send_aio(ctx, msg[["value"]], mode = 2L, timeout = 500))
@@ -215,16 +216,20 @@ nanotestaio(rek <- request(req$context, c(1+3i, 4+2i), send_mode = 2L, recv_mode
 nanotest(is.integer(reply(ctx, execute = identity, recv_mode = 3L, send_mode = "ra", timeout = 500)))
 nanotest(is.complex(call_aio(rek)[["data"]]))
 
-nanotest(is.list(nxt <- next_config(refhook = list(function(x) serialize(x, NULL), unserialize))))
+nanotest(is.pairlist(nxt <- next_config(refhook = list(function(x) serialize(x, NULL), unserialize))))
 nanotest(length(nxt) == 2L)
 nanotest(is.function(nxt[[1L]]))
 nanotest(is.integer(req$send(list(rep, ctx), mode = 3L, block = 500)))
 nanotest(typeof(recv(rep, block = 500)[[1L]]) == "externalptr")
+nanotest(is.integer(send(rep, new.env(), mode = 3L, block = 500)))
+nanotest(identical(next_config(nxt), nxt))
+nanotest(identical(next_config(as.pairlist(c("a", "b", "c"))), nxt))
 nanotest(is.list(next_config(NULL, mark = TRUE)))
 nanotestn(unlist(next_config()))
 
 nanotest(inherits(cv <- cv(), "conditionVariable"))
 nanotestp(cv)
+nanotestxp(cv2 <- cv())
 nanotest(!until(cv, 10L))
 nanotest(!until(cv, 10))
 nanotest(!until_(cv, 10L))
@@ -243,8 +248,10 @@ nanotest(recv(ctxn, block = 500))
 nanotestz(send(ctxn, TRUE, mode = 3L, block = 500))
 nanotestz(reap(ctxn))
 nanotest(reap(ctxn) == 7L)
-nanotestz(pipe_notify(rep, cv, add = TRUE, remove = TRUE, flag = TRUE))
-nanotestz(pipe_notify(req$socket, cv = cv, add = FALSE, remove = TRUE, flag = FALSE))
+nanotestz(pipe_notify(rep, cv, add = TRUE, flag = TRUE))
+nanotestz(pipe_notify(rep, cv, remove = TRUE, flag = tools::SIGCONT))
+nanotestz(pipe_notify(req$socket, cv = cv, add = TRUE))
+nanotestz(pipe_notify(req$socket, cv = cv, cv2 = cv2, remove = TRUE, flag = tools::SIGCONT))
 nanotesterr(request_signal(err, "test", cv = cv), "valid")
 nanotesterr(recv_aio_signal(err, cv = cv, timeout = 500))
 nanotesterr(request_signal(.context(req$socket), "test", cv = err), "valid")
@@ -258,6 +265,7 @@ nanotesterr(cv_reset(err), "valid")
 nanotesterr(cv_signal(err), "valid")
 nanotesterr(pipe_notify(err, cv), "valid Socket")
 nanotesterr(pipe_notify(rep, err), "valid Condition Variable")
+nanotesterr(pipe_notify(rep, cv, err), "valid Condition Variable")
 nanotestz(req$context_close())
 nanotestn(req$context_close)
 nanotestz(req$close())
@@ -270,8 +278,8 @@ nanotestz(close(rep))
 
 nanotestnano(pub <- nano("pub", listen = "inproc://ps"))
 nanotestnano(sub <- nano("sub", dial = "inproc://ps", autostart = NA))
-nanotestxp(cv <- cv())
-nanotestxp(cv2 <- cv())
+nanotestz(cv_reset(cv))
+nanotestz(cv_reset(cv2))
 nanotestz(pipe_notify(pub$socket, cv, cv2, add = TRUE, remove = TRUE))
 nanotestnano(sub$opt(name = "sub:prefnew", value = FALSE))
 nanotest(!sub$opt(name = "sub:prefnew"))
@@ -298,6 +306,8 @@ nanotestz(cv_signal(cv))
 nanotest(cv_value(cv) == 1L)
 nanotest(wait_(cv3))
 nanotestxp(cv %~>% cv3)
+nanotesterr("a" %~>% cv3, "valid Condition Variable")
+nanotesterr(cv3 %~>% "a", "valid Condition Variable")
 
 nanotestnano(surv <- nano(protocol = "surveyor", listen = "inproc://sock1", dial = "inproc://sock2"))
 nanotestp(surv)
@@ -342,6 +352,7 @@ nanotestnn(ncurl("http://www.cam.ac.uk/", follow = FALSE, response = list("date"
 nanotestnn(ncurl("http://www.cam.ac.uk/", follow = TRUE))
 nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L))
 nanotestnn(ncurl("http://postman-echo.com/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server"), timeout = 3000))
+nanotest(is_error_value(ncurl("http")$data))
 haio <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = "server")
 nanotest(is.integer(call_aio(haio)$status))
 haio <- ncurl_aio("https://trin.cam.ac.uk/", convert = FALSE, response = list("server"))
@@ -354,13 +365,12 @@ put2 <- ncurl_aio("http://postman-echo.com/put", method = "PUT", headers = list(
 nanotestp(put2)
 nanotest(is.integer(call_aio(put2)$status))
 nanotest(is.null(stop_aio(put2)))
-haio <- ncurl("http")
-nanotest(is_error_value(haio$data))
 haio <- ncurl_aio("https://i.i")
 nanotest(is_error_value(call_aio(haio)$data))
 nanotestp(haio$data)
 ncaio <- ncurl_aio("https://shikokuchuo.net/nanonext/reference/figures/logo.png")
 nanotestw(is.raw(call_aio(ncaio)$data) || is_error_value(ncaio$data))
+nanotest(is_error_value(ncurl_aio("http")$data))
 sess <- ncurl_session("https://postman-echo.com/post", method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test", response = "server", timeout = 3000L)
 nanotestnn(sess)
 nanotest(is_error_value(sess) || length(transact(sess)) == 3L)
@@ -371,10 +381,14 @@ nanotest(is_error_value(sess) || length(transact(sess)) == 3L)
 nanotest(is_error_value(sess) || close(sess) == 0L)
 nanotesterr(transact(sess), "ncurlSession")
 nanotestw(is_error_value(ncurl_session("https://i")))
+nanotesterr(ncurl_aio("https://", tls = "wrong"), "valid TLS")
 nanotesterr(ncurl("https://www.cam.ac.uk/", tls = "wrong"), "valid TLS")
 nanotesterr(stream(dial = "wss://127.0.0.1:5555"))
+nanotesterr(stream(dial = "errorValue3"), "argument")
+nanotesterr(stream(dial = "inproc://notsup"), "Not supported")
 nanotesterr(stream(dial = "wss://127.0.0.1:5555", tls = "wrong"), "valid TLS")
 nanotesterr(stream(listen = "errorValue3"), "argument")
+nanotesterr(stream(listen = "inproc://notsup"), "Not supported")
 nanotesterr(stream(listen = "errorValue3", tls = "wrong"), "valid TLS")
 nanotesterr(stream(), "specify")
 
@@ -393,7 +407,8 @@ nanotest(all(nzchar(parse_url("wss://use:r@[::1]/path?q=1#name"))))
 nanotest(is.character(random()))
 nanotest(nchar(random(2)) == 4L)
 nanotest(length(random(4L, convert = FALSE)) == 4L)
-nanotesterr(random(1e4L), "54")
+nanotesterr(random(1025), "between 0 and 1024")
+nanotesterr(random(-1), "between 0 and 1024")
 nanotesterr(random("test"), "integer")
 nanotesterr(parse_url("tcp:/"), "argument")
 for (i in c(100:103, 200:208, 226, 300:308, 400:426, 428:431, 451, 500:511))
@@ -411,7 +426,7 @@ if (length(s)) {
   nanotest(is.integer(send(s, "message2", block = FALSE)))
   nanotestnn(recv(s, mode = 9L, block = 100))
   nanotest(is.integer(send(s, 2L, block = 500)))
-  nanotestaio(sr <- recv_aio(s, mode = "integer", timeout = 500L, n = 8192L))
+  nanotestaio(sr <- recv_aio(s, mode = "i", timeout = 500L, n = 8192L))
   nanotestw(length(call_aio(sr)[["data"]]))
   nanotestn(stop_aio(sr))
   nanotestaio(ss <- send_aio(s, "async", timeout = 500L))
@@ -520,14 +535,17 @@ nanotest(is.integer(base64dec(base64enc(c(1L, 2L)), convert = NA)))
 nanotesterr(base64dec("__"), "not valid base64")
 nanotesterr(base64dec(404), "not valid base64")
 
-file <- tempfile()
 pem <- "-----BEGIN CERTIFICATE----- -----END CERTIFICATE-----"
-cat(pem, file = file)
-nanotesterr(tls_config(client = file), "Cryptographic error")
-nanotesterr(tls_config(server = file), "Cryptographic error")
+test_tls <- function(pem) {
+  file <- tempfile()
+  on.exit(unlink(file))
+  cat(pem, file = file)
+  nanotesterr(tls_config(client = file), "Cryptographic error")
+  nanotesterr(tls_config(server = file), "Cryptographic error")
+}
+nanotest(test_tls(pem = pem))
 nanotesterr(tls_config(client = c(pem, pem)), "Cryptographic error")
 nanotesterr(tls_config(server = c(pem, pem)), "Cryptographic error")
-unlink(file)
 nanotest(is.list(cert <- write_cert(cn = "127.0.0.1")))
 nanotest(length(cert) == 2L && is.character(cert[[1L]]))
 nanotest(identical(names(cert), c("server", "client")))
@@ -535,17 +553,17 @@ nanotestxp(tls <- tls_config(client = cert$client))
 nanotest(inherits(tls, "tlsConfig"))
 nanotestp(tls)
 nanotest(is_error_value(ncurl("https://www.cam.ac.uk/", tls = tls)$status))
-haio <- ncurl_aio("https://www.cam.ac.uk/", tls = tls)
-nanotest(is_error_value(call_aio(haio)$status))
+nanotest(is_error_value(call_aio(ncurl_aio("https://www.cam.ac.uk/", tls = tls))$data))
 nanotesterr(ncurl_session("https://www.cam.ac.uk/", tls = cert$client), "not a valid TLS")
 nanotestw(typeof(b <- ncurl_session("https://www.cam.ac.uk/", tls = tls)) == "externalptr" || is_error_value(b))
 nanotest(if (!is_error_value(b)) is_error_value(transact(b)[["headers"]]) else TRUE)
 nanotestxp(s <- socket(listen = "tls+tcp://127.0.0.1:5556", tls = tls_config(server = cert$server)))
 nanotestxp(s1 <- socket(dial = "tls+tcp://127.0.0.1:5556", tls = tls))
-nanotestw(dial(s, url = "tls+tcp://[", tls = tls, error = FALSE) > 0)
-nanotestw(listen(s, url = "tls+tcp://[", tls = tls, error = FALSE) > 0)
+nanotestw(dial(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
+nanotestw(listen(s, url = "tls+tcp://.", tls = tls, error = FALSE) > 0)
 nanotestz(close(s1))
 nanotestz(close(s))
 nanotestxp(tls <- tls_config())
+nanotesterr(stream(dial = "wss://127.0.0.1:5555", textframes = TRUE, tls = tls))
 nanotestxp(s <- socket(listen = "wss://127.0.0.1:5557", tls = tls))
 nanotestz(close(s))
